Cabal-syntax
Safe HaskellSafe-Inferred
LanguageHaskell2010

Distribution.Parsec.Warning

Synopsis

Documentation

data PWarning Source #

Parser warning.

Instances

Instances details
Binary PWarning Source # 
Instance details

Defined in Distribution.Parsec.Warning

Methods

put :: PWarning -> Put #

get :: Get PWarning #

putList :: [PWarning] -> Put #

NFData PWarning Source # 
Instance details

Defined in Distribution.Parsec.Warning

Methods

rnf :: PWarning -> () #

Generic PWarning Source # 
Instance details

Defined in Distribution.Parsec.Warning

Associated Types

type Rep PWarning 
Instance details

Defined in Distribution.Parsec.Warning

type Rep PWarning = D1 ('MetaData "PWarning" "Distribution.Parsec.Warning" "Cabal-syntax-3.18.1.0-inplace" 'False) (C1 ('MetaCons "PWarning" 'PrefixI 'True) (S1 ('MetaSel ('Just "pwarningType") 'NoSourceUnpackedness 'SourceStrict 'DecidedStrict) (Rec0 PWarnType) :*: (S1 ('MetaSel ('Just "pwarningPosition") 'NoSourceUnpackedness 'SourceStrict 'DecidedStrict) (Rec0 Position) :*: S1 ('MetaSel ('Just "pwarningMessage") 'NoSourceUnpackedness 'SourceStrict 'DecidedStrict) (Rec0 String))))

Methods

from :: PWarning -> Rep PWarning x #

to :: Rep PWarning x -> PWarning #

Show PWarning Source # 
Instance details

Defined in Distribution.Parsec.Warning

Eq PWarning Source # 
Instance details

Defined in Distribution.Parsec.Warning

Ord PWarning Source # 
Instance details

Defined in Distribution.Parsec.Warning

type Rep PWarning Source # 
Instance details

Defined in Distribution.Parsec.Warning

type Rep PWarning = D1 ('MetaData "PWarning" "Distribution.Parsec.Warning" "Cabal-syntax-3.18.1.0-inplace" 'False) (C1 ('MetaCons "PWarning" 'PrefixI 'True) (S1 ('MetaSel ('Just "pwarningType") 'NoSourceUnpackedness 'SourceStrict 'DecidedStrict) (Rec0 PWarnType) :*: (S1 ('MetaSel ('Just "pwarningPosition") 'NoSourceUnpackedness 'SourceStrict 'DecidedStrict) (Rec0 Position) :*: S1 ('MetaSel ('Just "pwarningMessage") 'NoSourceUnpackedness 'SourceStrict 'DecidedStrict) (Rec0 String))))

data PWarningWithSource src Source #

Constructors

PWarningWithSource 

Fields

Instances

Instances details
Functor PWarningWithSource Source # 
Instance details

Defined in Distribution.Parsec.Warning

Generic (PWarningWithSource src) Source # 
Instance details

Defined in Distribution.Parsec.Warning

Associated Types

type Rep (PWarningWithSource src) 
Instance details

Defined in Distribution.Parsec.Warning

type Rep (PWarningWithSource src) = D1 ('MetaData "PWarningWithSource" "Distribution.Parsec.Warning" "Cabal-syntax-3.18.1.0-inplace" 'False) (C1 ('MetaCons "PWarningWithSource" 'PrefixI 'True) (S1 ('MetaSel ('Just "pwarningSource") 'NoSourceUnpackedness 'SourceStrict 'DecidedStrict) (Rec0 (PSource src)) :*: S1 ('MetaSel ('Just "pwarning") 'NoSourceUnpackedness 'SourceStrict 'DecidedStrict) (Rec0 PWarning)))
Show src => Show (PWarningWithSource src) Source # 
Instance details

Defined in Distribution.Parsec.Warning

Eq src => Eq (PWarningWithSource src) Source # 
Instance details

Defined in Distribution.Parsec.Warning

Ord src => Ord (PWarningWithSource src) Source # 
Instance details

Defined in Distribution.Parsec.Warning

type Rep (PWarningWithSource src) Source # 
Instance details

Defined in Distribution.Parsec.Warning

type Rep (PWarningWithSource src) = D1 ('MetaData "PWarningWithSource" "Distribution.Parsec.Warning" "Cabal-syntax-3.18.1.0-inplace" 'False) (C1 ('MetaCons "PWarningWithSource" 'PrefixI 'True) (S1 ('MetaSel ('Just "pwarningSource") 'NoSourceUnpackedness 'SourceStrict 'DecidedStrict) (Rec0 (PSource src)) :*: S1 ('MetaSel ('Just "pwarning") 'NoSourceUnpackedness 'SourceStrict 'DecidedStrict) (Rec0 PWarning)))

data PSource src Source #

The source of a parse error

Constructors

PKnownSource src 
PUnknownSource 

Instances

Instances details
Functor PSource Source # 
Instance details

Defined in Distribution.Parsec.Source

Methods

fmap :: (a -> b) -> PSource a -> PSource b #

(<$) :: a -> PSource b -> PSource a #

Binary src => Binary (PSource src) Source # 
Instance details

Defined in Distribution.Parsec.Source

Methods

put :: PSource src -> Put #

get :: Get (PSource src) #

putList :: [PSource src] -> Put #

NFData src => NFData (PSource src) Source # 
Instance details

Defined in Distribution.Parsec.Source

Methods

rnf :: PSource src -> () #

Generic (PSource src) Source # 
Instance details

Defined in Distribution.Parsec.Source

Associated Types

type Rep (PSource src) 
Instance details

Defined in Distribution.Parsec.Source

type Rep (PSource src) = D1 ('MetaData "PSource" "Distribution.Parsec.Source" "Cabal-syntax-3.18.1.0-inplace" 'False) (C1 ('MetaCons "PKnownSource" 'PrefixI 'False) (S1 ('MetaSel ('Nothing :: Maybe Symbol) 'NoSourceUnpackedness 'NoSourceStrictness 'DecidedLazy) (Rec0 src)) :+: C1 ('MetaCons "PUnknownSource" 'PrefixI 'False) (U1 :: Type -> Type))

Methods

from :: PSource src -> Rep (PSource src) x #

to :: Rep (PSource src) x -> PSource src #

Show src => Show (PSource src) Source # 
Instance details

Defined in Distribution.Parsec.Source

Methods

showsPrec :: Int -> PSource src -> ShowS #

show :: PSource src -> String #

showList :: [PSource src] -> ShowS #

Eq src => Eq (PSource src) Source # 
Instance details

Defined in Distribution.Parsec.Source

Methods

(==) :: PSource src -> PSource src -> Bool #

(/=) :: PSource src -> PSource src -> Bool #

Ord src => Ord (PSource src) Source # 
Instance details

Defined in Distribution.Parsec.Source

Methods

compare :: PSource src -> PSource src -> Ordering #

(<) :: PSource src -> PSource src -> Bool #

(<=) :: PSource src -> PSource src -> Bool #

(>) :: PSource src -> PSource src -> Bool #

(>=) :: PSource src -> PSource src -> Bool #

max :: PSource src -> PSource src -> PSource src #

min :: PSource src -> PSource src -> PSource src #

type Rep (PSource src) Source # 
Instance details

Defined in Distribution.Parsec.Source

type Rep (PSource src) = D1 ('MetaData "PSource" "Distribution.Parsec.Source" "Cabal-syntax-3.18.1.0-inplace" 'False) (C1 ('MetaCons "PKnownSource" 'PrefixI 'False) (S1 ('MetaSel ('Nothing :: Maybe Symbol) 'NoSourceUnpackedness 'NoSourceStrictness 'DecidedLazy) (Rec0 src)) :+: C1 ('MetaCons "PUnknownSource" 'PrefixI 'False) (U1 :: Type -> Type))

data PWarnType Source #

Type of parser warning. We do classify warnings.

Different application may decide not to show some, or have fatal behaviour on others

Constructors

PWTOther

Unclassified warning

PWTUTF

Invalid UTF encoding

PWTBoolCase

true or false, not True or False

PWTVersionTag

there are version with tags

PWTNewSyntax

New syntax used, but no cabal-version: >= 1.2 specified

PWTOldSyntax

Old syntax used, and cabal-version >= 1.2 specified

PWTDeprecatedField 
PWTInvalidSubsection 
PWTUnknownField 
PWTUnknownSection 
PWTTrailingFields 
PWTExtraMainIs

extra main-is field

PWTExtraTestModule

extra test-module field

PWTExtraBenchmarkModule

extra benchmark-module field

PWTLexNBSP 
PWTLexBOM 
PWTLexTab 
PWTQuirkyCabalFile

legacy cabal file that we know how to patch

PWTDoubleDash

Double dash token, most likely it's a mistake - it's not a comment

PWTMultipleSingularField

e.g. name or version should be specified only once.

PWTBuildTypeDefault

Workaround for derive-package having build-type: Default. See https://github.com/haskell/cabal/issues/5020.

PWTVersionOperator

Version operators used (without cabal-version: 1.8)

PWTVersionWildcard

Version wildcard used (without cabal-version: 1.6)

PWTSpecVersion

Warnings about cabal-version format.

PWTEmptyFilePath

Empty filepath, i.e. literally ""

PWTInconsistentIndentation

sections contents (sections and fields) are indented inconsistently

PWTExperimental

Experimental feature

Instances

Instances details
Binary PWarnType Source # 
Instance details

Defined in Distribution.Parsec.Warning

NFData PWarnType Source # 
Instance details

Defined in Distribution.Parsec.Warning

Methods

rnf :: PWarnType -> () #

Bounded PWarnType Source # 
Instance details

Defined in Distribution.Parsec.Warning

Enum PWarnType Source # 
Instance details

Defined in Distribution.Parsec.Warning

Generic PWarnType Source # 
Instance details

Defined in Distribution.Parsec.Warning

Associated Types

type Rep PWarnType 
Instance details

Defined in Distribution.Parsec.Warning

type Rep PWarnType = D1 ('MetaData "PWarnType" "Distribution.Parsec.Warning" "Cabal-syntax-3.18.1.0-inplace" 'False) ((((C1 ('MetaCons "PWTOther" 'PrefixI 'False) (U1 :: Type -> Type) :+: (C1 ('MetaCons "PWTUTF" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTBoolCase" 'PrefixI 'False) (U1 :: Type -> Type))) :+: (C1 ('MetaCons "PWTVersionTag" 'PrefixI 'False) (U1 :: Type -> Type) :+: (C1 ('MetaCons "PWTNewSyntax" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTOldSyntax" 'PrefixI 'False) (U1 :: Type -> Type)))) :+: ((C1 ('MetaCons "PWTDeprecatedField" 'PrefixI 'False) (U1 :: Type -> Type) :+: (C1 ('MetaCons "PWTInvalidSubsection" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTUnknownField" 'PrefixI 'False) (U1 :: Type -> Type))) :+: ((C1 ('MetaCons "PWTUnknownSection" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTTrailingFields" 'PrefixI 'False) (U1 :: Type -> Type)) :+: (C1 ('MetaCons "PWTExtraMainIs" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTExtraTestModule" 'PrefixI 'False) (U1 :: Type -> Type))))) :+: (((C1 ('MetaCons "PWTExtraBenchmarkModule" 'PrefixI 'False) (U1 :: Type -> Type) :+: (C1 ('MetaCons "PWTLexNBSP" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTLexBOM" 'PrefixI 'False) (U1 :: Type -> Type))) :+: ((C1 ('MetaCons "PWTLexTab" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTQuirkyCabalFile" 'PrefixI 'False) (U1 :: Type -> Type)) :+: (C1 ('MetaCons "PWTDoubleDash" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTMultipleSingularField" 'PrefixI 'False) (U1 :: Type -> Type)))) :+: ((C1 ('MetaCons "PWTBuildTypeDefault" 'PrefixI 'False) (U1 :: Type -> Type) :+: (C1 ('MetaCons "PWTVersionOperator" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTVersionWildcard" 'PrefixI 'False) (U1 :: Type -> Type))) :+: ((C1 ('MetaCons "PWTSpecVersion" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTEmptyFilePath" 'PrefixI 'False) (U1 :: Type -> Type)) :+: (C1 ('MetaCons "PWTInconsistentIndentation" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTExperimental" 'PrefixI 'False) (U1 :: Type -> Type))))))
Show PWarnType Source # 
Instance details

Defined in Distribution.Parsec.Warning

Eq PWarnType Source # 
Instance details

Defined in Distribution.Parsec.Warning

Ord PWarnType Source # 
Instance details

Defined in Distribution.Parsec.Warning

type Rep PWarnType Source # 
Instance details

Defined in Distribution.Parsec.Warning

type Rep PWarnType = D1 ('MetaData "PWarnType" "Distribution.Parsec.Warning" "Cabal-syntax-3.18.1.0-inplace" 'False) ((((C1 ('MetaCons "PWTOther" 'PrefixI 'False) (U1 :: Type -> Type) :+: (C1 ('MetaCons "PWTUTF" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTBoolCase" 'PrefixI 'False) (U1 :: Type -> Type))) :+: (C1 ('MetaCons "PWTVersionTag" 'PrefixI 'False) (U1 :: Type -> Type) :+: (C1 ('MetaCons "PWTNewSyntax" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTOldSyntax" 'PrefixI 'False) (U1 :: Type -> Type)))) :+: ((C1 ('MetaCons "PWTDeprecatedField" 'PrefixI 'False) (U1 :: Type -> Type) :+: (C1 ('MetaCons "PWTInvalidSubsection" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTUnknownField" 'PrefixI 'False) (U1 :: Type -> Type))) :+: ((C1 ('MetaCons "PWTUnknownSection" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTTrailingFields" 'PrefixI 'False) (U1 :: Type -> Type)) :+: (C1 ('MetaCons "PWTExtraMainIs" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTExtraTestModule" 'PrefixI 'False) (U1 :: Type -> Type))))) :+: (((C1 ('MetaCons "PWTExtraBenchmarkModule" 'PrefixI 'False) (U1 :: Type -> Type) :+: (C1 ('MetaCons "PWTLexNBSP" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTLexBOM" 'PrefixI 'False) (U1 :: Type -> Type))) :+: ((C1 ('MetaCons "PWTLexTab" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTQuirkyCabalFile" 'PrefixI 'False) (U1 :: Type -> Type)) :+: (C1 ('MetaCons "PWTDoubleDash" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTMultipleSingularField" 'PrefixI 'False) (U1 :: Type -> Type)))) :+: ((C1 ('MetaCons "PWTBuildTypeDefault" 'PrefixI 'False) (U1 :: Type -> Type) :+: (C1 ('MetaCons "PWTVersionOperator" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTVersionWildcard" 'PrefixI 'False) (U1 :: Type -> Type))) :+: ((C1 ('MetaCons "PWTSpecVersion" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTEmptyFilePath" 'PrefixI 'False) (U1 :: Type -> Type)) :+: (C1 ('MetaCons "PWTInconsistentIndentation" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "PWTExperimental" 'PrefixI 'False) (U1 :: Type -> Type))))))